package engine;

/**
 * <p>This class contains all the computations for ALTMMR.</p>
 */
public class Engine
{
    public static EngineOutput calc(EngineInput engineInput)
    {
        EngineOutput engineOutput = new EngineOutput();

        // Local variables for computations

        double[] m_barray;
        double[] m_varray;
        double[] m_sysqrarray;
        double[] m_sxsqrarray;
        double[] m_rxysqrarray;
        double[] m_errvar;
        double[] m_stderrsqr;
        double[] m_invstderrsqr;
        double[] m_weight;
        double[][] Rst;
        double[] m_95chi;
        double[] m_99chi;
        double[] m_chi;
        double[] ratios;
        double dEVRatio;


        int m_k = engineInput.getSize();
        double[] m_narray = engineInput.getMn();
        double[] m_sxarray = engineInput.getMsx();
        double[] m_syarray = engineInput.getMsy();
        double[] m_rxyarray = engineInput.getMrxy();

        double[] m_t = new double[m_k];

        m_varray = new double[m_k];
        m_sysqrarray = new double[m_k];
        m_sxsqrarray = new double[m_k];
        m_rxysqrarray = new double[m_k];
        m_barray = new double[m_k];
        m_errvar     = new double[m_k];

        double[] m_stderr = new double[m_k];

            //number of errvar ratios to compare
            int compares = 0;
            for (int i=1;i<m_k;i++) {
                compares = compares + i;
            }
            ratios = new double[compares];
            m_stderrsqr  = new double[m_k];
            m_invstderrsqr = new double[m_k];
            m_weight = new double[m_k];
            Rst = new double[2][4];

            //  create arrays of critical values (c) for chi-square
            //  distribution for alpha = .05 and .01; v (deg of freedom)
            //  are k-1.  Rows in array are v+1--i.e. m_95chi[0] is the
            //  critical value for v = 1, alpha = .05.
            //  Source, p. 577, Walpole & Myers (1985)
            m_95chi = new double[16];
            m_95chi[0]=3.841;
            m_95chi[1]=5.991;
            m_95chi[2]=7.81473;
            m_95chi[3]=9.488;
            m_95chi[4]=11.070;
            m_95chi[5]=12.592;
            m_95chi[6]=14.067;
            m_95chi[7]=15.507;
            m_95chi[8]=16.919;
            m_95chi[9]=18.307;
            m_95chi[10]=19.675;
            m_95chi[11]=21.026;
            m_95chi[12]=22.362;
            m_95chi[13]=23.685;
            m_95chi[14]=24.996;
            m_95chi[15]=26.296;
            m_99chi = new double[16];
            m_99chi[0]=6.635;
            m_99chi[1]=9.210;
            m_99chi[2]=11.345;
            m_99chi[3]=13.277;
            m_99chi[4]=15.086;
            m_99chi[5]=16.812;
            m_99chi[6]=18.475;
            m_99chi[7]=20.090;
            m_99chi[8]=21.666;
            m_99chi[9]=23.209;
            m_99chi[10]=24.725;
            m_99chi[11]=26.217;
            m_99chi[12]=27.688;
            m_99chi[13]=29.141;
            m_99chi[14]=30.578;
            m_99chi[15]=32.000;

            m_chi = new double[4];
            double suminvstderrsqr = 0.0;

            //  Calculate squared values and put into arrays

            for (int counter = 0; counter < m_k; counter++) {
                m_sysqrarray[counter] = m_syarray[counter] * m_syarray[counter];
                m_sxsqrarray[counter] = m_sxarray[counter] * m_sxarray[counter];
                m_rxysqrarray[counter] = m_rxyarray[counter] * m_rxyarray[counter];
                m_errvar[counter] = m_sysqrarray[counter] * (1 - m_rxysqrarray[counter]);

                //calc standard error and squared standard error (Sbisquared) =
                //((1-rsqr)*Sysqr)/(n-2)*Sxsqr) from app C

                m_stderrsqr[counter] = ((1-m_rxysqrarray[counter]) *
                m_sysqrarray[counter])/((m_narray[counter] - 2)* m_sxsqrarray[counter]);
                m_stderr[counter] = Math.pow(m_stderrsqr[counter],.5);

                //make array of inverse squared standard errors and sum it
                m_invstderrsqr[counter] = 1 / m_stderrsqr[counter];
                suminvstderrsqr = suminvstderrsqr+m_invstderrsqr[counter];

                //make array of regression weights (b) called m_barray
                double varianceratio = m_sysqrarray[counter] / m_sxsqrarray[counter];
                m_barray[counter] = m_rxyarray[counter] * (Math.sqrt(varianceratio));
            }

            double highratio = -1.0;
            if(m_k == 2) {
                if(m_errvar[0] > m_errvar[1]) {
                        dEVRatio=m_errvar[0]/m_errvar[1];
                } else {
                        dEVRatio=m_errvar[1]/m_errvar[0];
                }
                ratios[0] = dEVRatio;

                if (dEVRatio > 1.5) {
                        dEVRatio = (MathUtil.round(dEVRatio,2));
                        engineOutput.setThumb(" is NOT met. (The Error Variance ratio is 1:" + String.valueOf(dEVRatio) + ")");
                } else {
                        dEVRatio = (MathUtil.round(dEVRatio,2));
                        engineOutput.setThumb(" is met. (The Error Variance ratio is 1:" + String.valueOf(dEVRatio) + ")");
                }

            } else {
                dEVRatio = 0;
            }

            if (m_k > 2) {
                int k = 0;
                for (int j=0;j<(m_k-1);j++) {
                    for(int i=(j+1);i<m_k;i++) {
                        if(m_errvar[j] > m_errvar[i]) {
                            ratios[k]=m_errvar[j]/m_errvar[i];
                        } else {
                            ratios[k]=m_errvar[i]/m_errvar[j];
                        }

                        if (ratios[k] > highratio) {
                            highratio = ratios[k];
                        }
                        k++;
                    }
                }

                dEVRatio = highratio;
                highratio = (MathUtil.round(highratio,2));

                if (dEVRatio > 1.5) {
                    engineOutput.setThumb(" is NOT met. (The highest Error Variance ratio is 1:" + String.valueOf(highratio) + ")");
                } else {
                    engineOutput.setThumb(" is met.  (The highest Error Variance ratio is 1:" + String.valueOf(highratio) + ")");
                }
             }


            //*************Calculate Bartlett's Stat**************
            //fill m_varray & calculate necessary sums
            double sumv = 0.0;//sum of degrees of freedom
            double sumvinv = 0;//sum of inverse of df
            double sumverrvar = 0.0;//  sum of df *errorvariances
            double sumvlogerrvar = 0.0; // sum of df * log of errvar

            for (int i = 0; i < m_k; i++) {
                m_varray[i] = m_narray[i] - 1;
                sumv = sumv + m_varray[i];
                sumvinv = sumvinv + (1/m_varray[i]);
                sumverrvar = sumverrvar + (m_errvar[i]*m_varray[i]);
                sumvlogerrvar = sumvlogerrvar + (m_varray[i] * Math.log(m_errvar[i]));
            }

            //Calculate M
            double numerator1 = 0.0;
            double denominator1 = 0.0;
            double M = 0.0;

            numerator1 = sumv * Math.log(sumverrvar/sumv)- sumvlogerrvar;
            denominator1 =(1/(3*(m_k-1))* (sumvinv - (1/sumv))) + 1;
            M = numerator1 / denominator1;

            double pM = (MathUtil.Chi(m_k,M));//get p-value for this M
            //  Output M and associated p-value

            String sp;
            if (pM < .00001) {
                sp = " < .00001";
            } else {
                pM = (MathUtil.round(pM,4));
                sp = " =  " + String.valueOf(pM);
            }

            //create string 'homo' and 'homo2' depending on errvar ratio and p value
            if (dEVRatio > 1.5 & pM < .05) {
                engineOutput.setHomo("Since both statistics indicate heterogeneous error variance, a necessary assumption of");
                engineOutput.setHomo2("MMR is NOT met, and the alternative statistics below are more accurate indicators of differential slopes.");
            }

            if (dEVRatio < 1.5 & pM > .05) {
                engineOutput.setHomo("Both statistics indicate that the homogeneity of sub-group error variance assumption is met.");
                engineOutput.setHomo2("Thus, MMR may be the best differential slopes assessment technique for this data.");
            }

            if (dEVRatio < 1.5 & pM < .05) {
                engineOutput.setHomo("These conflicting results may or may not indicate violation of the homogeneity of error");
                engineOutput.setHomo2("variance assumption for MMR. However, Bartlett's test is susceptible to non-normality, and may be over-conservative.");
            }

            if (dEVRatio > 1.5 & pM > .05) {
                engineOutput.setHomo("These conflicting results may or may not indicate violation of the homogeneity of error");
                engineOutput.setHomo2("variance assumption for MMR. However,the rule of thumb was derived from extensive simulation.");
            }

            M = (MathUtil.round(M,4));

            if (pM <.05) {
                engineOutput.setBartlett("indicates heterogeneous error variance (M = " + String.valueOf(M) +	", p " + sp + ").");
            } else {
                engineOutput.setBartlett("indicates homogeneous error variance (M = " + String.valueOf(M) +	", p " + sp + ").");
            }


            //*************CALCULATE JAMES'S STAT*****************
            //calculate weight for each regression weight (James) and the value for b+

            double bplus = 0.0;
            for (int c2=0; c2 < m_k; c2++) {
                m_varray[c2] = m_narray[c2] - 2; // v is different for James and Alexander's
                m_weight[c2] = m_invstderrsqr[c2] / suminvstderrsqr;
                bplus = bplus + m_weight[c2] * m_barray[c2];
            }

            //calculate U  (James's Stat)
            double U = 0.0;
            for (int c3=0;c3<m_k;c3++) {
                U = U + (m_barray[c3]-bplus)*(m_barray[c3]-bplus)/m_stderrsqr[c3];
            }

            // calculate adjusted critical value for U
            // note:  U is not referenced to the chi-square dist.
            // the following calculations begin with the
            // critical value given dF and alpha, and then
            // adjusts it--this is the key to the approximation

            double dt = 0.0;
            double ds = 0.0;
            int S = 0;
            double currentR = 0.0;

            //create an array of arrays (matrix) where
            // Rst is R[s+1][t]   s is column, t is row(step 6, AppC)
            //  verified R10 and R23

            for (int s=0; s < 2; s++) {  // column loop
                S = s + 1; // S for James notation, s for array element
                ds = S; // arg for Math.pow must be double

                for (int t=0; t < 4; t++) {//row loop
                    currentR = 0.0;

                    for (int i=0; i<m_k; i++) {// sum of w/v
                        dt = t;
                        currentR = currentR + Math.pow(m_weight[i], dt) / Math.pow(m_varray[i], ds);
                    }// close sum loop

                    Rst[s][t] = currentR;
                } // close row loop

            }// close column loop

            //Step 7.  Compute chi 2,4,6,8 values and put in array
            // c is the critical value for v (i.e. k-1) degrees of freedom
            // retrieved from the appropriate m_95chi or m_99chi array
            //get table p-value

            double c;
            if (engineInput.getMAlpha() == 0.01) {
                c = m_99chi[m_k-2];// m_99chi[0] is for 1 degree of freedom
            } else {
                c = m_95chi[m_k-2];
            }

            double product=1.0;
            double numerator = 0.0;
            for (int s2=1;s2<5;s2++) {
                numerator = Math.pow(c,s2);//c to the s2 power
                for (int s3=1;s3<s2+1;s3++) {
                    product = product * (m_k + 2 * s3 - 3);
                }
                //fill array m_chi, where [0] is where
                // s = 1 or 2s = 2
                m_chi[s2-1] = numerator/product; // hand calcs verified 2.605,4.07,4.56,3.95
                product = 1.0;
            }

            //step 8:  calculate T
            double T = 0.0;
            for (int kcount=0;kcount < m_k; kcount++) {
                T = T + (Math.pow(1-m_weight[kcount],2) / m_varray[kcount]);
            }

            //Step 9:  calculate adjusted critical value or h(alpha)
            double a = 0.0;
            double hcrit = 0.0;
            double brack = 0.0; //calcs inside bracket of eqn9
            double brack1 = 0.0;
            double brack2 = 0.0;
            double brack3 = 0.0;
            double brace = 0.0; //calcs inside brace of eqn 9
            double e = 0.0;
            double e1 = 0.0;
            double f = 0.0;
            double g = 0.0;
            double h = 0.0;
            double i = 0.0;
            double j = 0.0;

            a = c+.5*(3*m_chi[1]+m_chi[0])*T; //a
            brack1=8*Rst[1][3]-10*Rst[1][2]+4*Rst[1][1]-6*Math.pow(Rst[0][2],2)+8*Rst[0][2]*Rst[0][1]-4*Math.pow(Rst[0][1],2);
            brack2=(2*Rst[1][3]-4*Rst[1][2]+2*Rst[1][1]-2*(Math.pow(Rst[0][2],2))+4*Rst[0][2]*Rst[0][1]-2*(Math.pow(Rst[0][1],2)))*(m_chi[0]-1);
            brack3=(.25*(4*Rst[0][2]*Rst[0][1]-Math.pow(Rst[0][2],2)-2*Rst[0][2]*Rst[0][0]-4*Math.pow(Rst[0][1],2)+4*Rst[0][1]*Rst[0][0]-Math.pow(Rst[0][0],2)))*(3*m_chi[1]-2*m_chi[0]-1);
            brack = brack1 + brack2 + brack3;
            e1 = .0625*(Math.pow(3*m_chi[1]+m_chi[0],2))*(1-(m_k-3))/c*Math.pow(T,2);
            e = (.5*(3*m_chi[1]+m_chi[0]));
            f = (Rst[1][3]-3*Rst[1][2]+3*Rst[1][1]-Rst[1][0])*(5*m_chi[2]+2*m_chi[1]+m_chi[0]);
            g = .1875*((Math.pow(Rst[0][2],2)-4*Rst[1][3]+6*Rst[1][2]-4*Rst[1][1]+Rst[1][0]))*(35*m_chi[3]+15*m_chi[2]+9*m_chi[1]+5*m_chi[0]);
            h = .0625*(4*Rst[1][1]-2*Rst[1][2]-Rst[1][0]+2*	Rst[0][2]*Rst[0][0]-4*Rst[0][1]*Rst[0][0]+Math.pow(Rst[0][0],2))*(9*m_chi[3]-3*m_chi[2]-5*m_chi[1]-m_chi[0]);
            i = .25*(Math.pow(Rst[0][1],2)-Rst[1][2])*(27*m_chi[3]+3*m_chi[2]+m_chi[1]+m_chi[0]);
            j = .25*(Rst[1][3]-Rst[0][2]*Rst[0][1])*(45*m_chi[3]+9*m_chi[2]+7*m_chi[1]+3*m_chi[0]);
            brace = e1+e*brack+f+g+h+i+j;
            hcrit = a + brace;

            //repaint();

            //Results for James Stat
            U = (MathUtil.round(U,4));
            hcrit = (MathUtil.round(hcrit,4));

            if (U < hcrit) {
                hcrit = (MathUtil.round(hcrit,4));//round after comparing to U
                engineOutput.setJames(" indicates no differential slopes. (p > " + String.valueOf(engineInput.getMAlpha()) + ") " + "U = " + String.valueOf(U) + ", and U(critical) = " + String.valueOf(hcrit));
            } else {
                hcrit = (MathUtil.round(hcrit,4));//round after comparing to U
                engineOutput.setJames(" indicates differential slopes. (p < " + String.valueOf(engineInput.getMAlpha()) + ") " + "U = " + String.valueOf(U) + ", and U(critical) = " + String.valueOf(hcrit));
            }

            //Calculate Alexander's A
            double A = 0.0;
            double z = 0.0;
            double z1;
            double z2;
            double z2a;
            double z2b;
            double a1;
            double b1;
            double c1;
            double c1a;

            for(int i5 = 0; i5 < m_k; i5++) {
                m_t[i5] = (m_barray[i5] - bplus)/m_stderr[i5];
                a1 = m_varray[i5] - 0.5;
                b1 = 48 * Math.pow(a1,2);
                c1a = Math.log(1 + Math.pow(m_t[i5], 2)/m_varray[i5]);
                c1 = Math.pow(a1*c1a, .5);
                z1 = (Math.pow(c1,3) + 3*c1)/b1;
                z2a = 4*Math.pow(c1, 7) + 33*Math.pow(c1, 5) + 240 * Math.pow(c1, 3) + 855*c1;
                z2b = 10*Math.pow(b1,2)+8*b1*Math.pow(c1,4)+1000*b1;
                z2 = z2a/z2b;
                z = c1 + z1 - z2;
                A = A + Math.pow(z,2);
             }

            double pA = (MathUtil.Chi(m_k,A));//get p-value for this A
            double pArnd = (MathUtil.round(pA,4));
            double Arnd = (MathUtil.round(A,4));


            if(pA < .05) {
                engineOutput.setAlex(" indicates differential slopes! (A = " + String.valueOf(Arnd) + ", p = " + String.valueOf(pArnd) + ")");
            }

            if(pA >= .05 & pA != 1.0) {
                engineOutput.setAlex(" indicates no evidence of differential slopes! (A = " + String.valueOf(Arnd) + ", p = " + String.valueOf(pArnd) + ")");
            }

            if (pA < .05 & U > hcrit) {
                engineOutput.setMeaning("Thus, both statistics indicate that the regression slopes for the sub-groups are different.");
            }

            else {
                engineOutput.setMeaning("Thus, both statistics indicate that the regression slopes for the sub-groups are not statistically different.");
            }

        return engineOutput;
    }

}

