package ui;

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.Border;

/**
 * <p>This Frame contains the data for each Sample and called
 * "sample" number of times to take input data from the user.</p>
 *  */
public class InputDataFrame extends JDialog implements ActionListener
{
    JLabel lblTitle = new JLabel();
    private JLabel lblSubGroupSize = new JLabel();
    private JTextField txtSubGroupSize = new JTextField();
    
    private JTextPane lblPredictor = new JTextPane();
    private JTextField txtPredictor = new JTextField();
    
    private JTextPane lblCriterion = new JTextPane();
    private JTextField txtCriterion = new JTextField();
    
    private JTextPane lblCorrelation = new JTextPane();
    private JTextField txtCorrelation = new JTextField();
	
	JButton btnNext = new JButton();
    private JButton btnExit = new JButton();
    private JButton btnHelp = new JButton();
    
    // Contains all the entries of each subgroup input
    private ArrayList subgroupDataList = new ArrayList();

   // Listener collects the user entered input data
   // to perform the calculations.
    private SubGroupListener listener;

    public InputDataFrame(JFrame owner) {
		super(owner);

        this.getContentPane().setLayout(null);
        this.getContentPane().setBackground(Color.white);
        this.setSize(new Dimension(440, 340));
        this.setModal(true);
        
		Border empty = BorderFactory.createEmptyBorder();
        
        lblTitle.setText("Input Data for Sub-Group X");
        lblTitle.setBounds(new Rectangle(123, 10, 400, 30));
        lblTitle.setFont(new Font("Dialog", 1, 14));

        lblSubGroupSize.setText("Sub-Group Size (n)");
        lblSubGroupSize.setBounds(new Rectangle(20, 55, 135, 30));
        lblSubGroupSize.setFont(new Font("Dialog", 1, 12));
        txtSubGroupSize.setBounds(new Rectangle(230, 55, 105, 25));

		String str = "Predictor Std.deviation ( S";
		String sub = "x";
        lblPredictor.setBounds(new Rectangle(20, 100, 190, 35));
        lblPredictor.setBackground(Color.white);
		lblPredictor.setBorder(empty);
		lblPredictor.setEditable(false);
		lblPredictor.setFocusable(false);
		CompUtil.getTextPane(lblPredictor, str,sub);
		txtPredictor.setBounds(new Rectangle(230, 100, 105, 25));

               
		str = "Criterion Std.deviation ( S";
        sub = "y";
        lblCriterion.setBounds(new Rectangle(20, 140, 190, 35));
		lblCriterion.setBackground(Color.white);
		lblCriterion.setBorder(empty);
		lblCriterion.setEditable(false);
		lblCriterion.setFocusable(false);
		CompUtil.getTextPane(lblCriterion, str, sub);
		txtCriterion.setBounds(new Rectangle(230, 140, 105, 25));
    
        
		str = "Correlation for X and Y (r";
		sub = "xy";
        lblCorrelation.setBounds(new Rectangle(20, 180, 190, 35));
	   	lblCorrelation.setBackground(Color.white);
		lblCorrelation.setBorder(empty);
		lblCorrelation.setEditable(false);
		lblCorrelation.setFocusable(false);
		CompUtil.getTextPane(lblCorrelation, str, sub);
		txtCorrelation.setBounds(new Rectangle(230, 180, 105, 25));


        btnNext.setText("Next");
        btnNext.setActionCommand("Next");
        btnNext.setBounds(new Rectangle(95, 240, 90, 25));
        btnNext.setFont(new Font("Dialog", 1, 12));
        btnNext.addActionListener(this);

        btnExit.setText("Exit");
        btnExit.setActionCommand("Exit");
        btnExit.setBounds(new Rectangle(215, 240, 90, 25));
        btnExit.setFont(new Font("Dialog", 1, 12));
        btnExit.addActionListener(this);

        btnHelp.setText("Help");
        btnHelp.setActionCommand("Help");
        btnExit.setFont(new Font("Dialog", 1, 12));
        btnHelp.setBounds(new Rectangle(335, 240,90, 25));
        btnHelp.addActionListener(this);

        this.getContentPane().add(lblTitle, null);
        this.getContentPane().add(lblSubGroupSize, null);
        this.getContentPane().add(txtSubGroupSize, null);
        this.getContentPane().add(lblPredictor, null);
        this.getContentPane().add(txtPredictor, null);
        this.getContentPane().add(lblCriterion, null);
        this.getContentPane().add(txtCriterion, null);
        this.getContentPane().add(lblCorrelation, null);
        this.getContentPane().add(txtCorrelation, null);
        this.getContentPane().add(btnNext, null);
        this.getContentPane().add(btnExit, null);
        this.getContentPane().add(btnHelp, null);
    }

    public void actionPerformed(ActionEvent ae)
    {
        double sn, ssx, ssy, srxy;
        if(ae.getActionCommand().equals("Next")) {
        	try {
        		sn = Double.parseDouble(txtSubGroupSize.getText());
            	ssx = Double.parseDouble(txtPredictor.getText());
            	ssy = Double.parseDouble(txtCriterion.getText());
            	srxy = Double.parseDouble(txtCorrelation.getText());
           
            	listener.handleSubGroupData(sn, ssx, ssy, srxy);
            	listener.originalInput(txtSubGroupSize.getText(),
                                   txtPredictor.getText(),
                                   txtCriterion.getText(),
                                   txtCorrelation.getText());
            	this.hide();
            	this.dispose();
        	} catch(Exception e) {
        		JOptionPane.showMessageDialog(null, "Enter appropriate values.", 
					"Alert", JOptionPane.ERROR_MESSAGE);
					txtSubGroupSize.requestFocus();
        	}

        } else if(ae.getActionCommand().equals("Exit")) {
            this.hide();
            this.dispose();


        }  else if(ae.getActionCommand().equals("Help")) {

            String message = "The inputs required are :\n" +
                "1) the size of the group (e.g., 100 males),\n" +
                "2) the standard deviation of text scores for that sub-group, \n" +
                "3) the standard deviation of criterion scores (e.g., job performance,\n"+
                "4) the pearson correlation of thesr test scores to the criterion scores\n" +
                "again for each sub-group seperately.\n" +
                "For more detailed help, see ALTMMR homepage.";

            JOptionPane.showMessageDialog(this, message, "ALRMMR Help", JOptionPane.INFORMATION_MESSAGE);
        }
    }

    public ArrayList getInputData()
    {
        return subgroupDataList;
    }

    public void clearDialog()
    {
        txtSubGroupSize.setText("");
        txtPredictor.setText("");
        txtCriterion.setText("");
        txtCorrelation.setText("");
    }

    public void addSubGroupListener(SubGroupListener listener)
    {
        this.listener = listener;
    }
}
